
# Apollo Android

[![GitHub license](https://img.shields.io/badge/license-MIT-lightgrey.svg?maxAge=2592000)](https://raw.githubusercontent.com/apollographql/apollo-android/master/LICENSE) [![Join Spectrum](https://img.shields.io/badge/spectrum-join-orange)](https://spectrum.chat/apollo/apollo-android)
![CI](https://github.com/apollographql/apollo-android/workflows/CI/badge.svg)
[![GitHub release](https://img.shields.io/github/release/apollographql/apollo-android.svg)](https://github.com/apollographql/apollo-android/releases/latest)

Apollo Android is a GraphQL client that generates Java and Kotlin models from GraphQL queries. These models give you a type-safe API to work with GraphQL servers.  Apollo helps you keep your GraphQL query statements together, organized, and easy to access. 

This library is designed primarily with Android in mind, but you can use it in any Java/Kotlin app. 

## Features

* Java and Kotlin code generation
* Queries, Mutations and Subscriptions
* Reflection-free parsing of responses
* HTTP cache
* Normalized cache
* File uploads
* Custom scalar types
* Support for RxJava2, RxJava3, and Coroutines

## Getting started

If you are new to GraphQL, check out [the tutorial](https://www.apollographql.com/docs/android/tutorial/00-introduction/) that will guide you through building an Android app using Apollo, Kotlin and coroutines.

If you'd like to add Apollo Android to an existing project:

* [Get started with Kotlin](https://www.apollographql.com/docs/android/essentials/get-started-kotlin) shows how to add Apollo Android to a Kotlin project.
* [Get started with Java](https://www.apollographql.com/docs/android/essentials/get-started-java) shows how to add Apollo Android to a Java project.
* [Get started with Multiplatform (Experimental)](https://www.apollographql.com/docs/android/essentials/get-started-multiplatform) shows how to add Apollo Android to a Multiplatform project. This is still under heavy development and APIs may change without warning.


## Advanced topics

Check [the project website](https://www.apollographql.com/docs/android/) for in depth documentation about [caching](https://www.apollographql.com/docs/android/essentials/caching/), [plugin configuration](https://www.apollographql.com/docs/android/essentials/plugin-configuration/), [android](https://www.apollographql.com/docs/android/advanced/android/), [file upload](https://www.apollographql.com/docs/android/advanced/file-upload/), [coroutines](https://www.apollographql.com/docs/android/advanced/coroutines/), [rxjava2](https://www.apollographql.com/docs/android/advanced/rxjava2/), [rxjava3](https://www.apollographql.com/docs/android/advanced/rxjava3/), [persisted queries](https://www.apollographql.com/docs/android/advanced/persisted-queries/), [no runtime](https://www.apollographql.com/docs/android/advanced/no-runtime/), [migrations](https://www.apollographql.com/docs/android/essentials/migration/) and much more...

## IntelliJ Plugin

The [JS Graphql IntelliJ Plugin](https://jimkyndemeyer.github.io/js-graphql-intellij-plugin/) provides auto-completion, error highlighting, and go-to-definition functionality for your `.graphql` files. You can create a [`.graphqlconfig`](https://jimkyndemeyer.github.io/js-graphql-intellij-plugin/docs/developer-guide#working-with-graphql-endpoints-and-scratch-files) file to use GraphQL scratch files to work with your schema outside product code (such as to write temporary queries to test resolvers).

## Releases

The latest version is [ ![Download](https://api.bintray.com/packages/apollographql/android/apollo/images/download.svg) ](https://bintray.com/apollographql/android/apollo-gradle-plugin/_latestVersion)

Check the [changelog](https://github.com/apollographql/apollo-android/releases) for the release history. 

Releases are hosted on [Jcenter](https://jcenter.bintray.com/com/apollographql/apollo/) and [Maven Central](https://repo1.maven.org/maven2/com/apollographql/apollo/). The plugin is additionally hosted on the [Gradle Plugin Portal](https://plugins.gradle.org/plugin/com.apollographql.apollo) 


```groovy:title=build.gradle.kts
plugins {
  id("com.apollographql.apollo").version("x.y.z")
}

repositories {
  jcenter()
}

dependencies {
  implementation("com.apollographql.apollo:apollo-runtime:x.y.z")

  // optional: if you want to use the normalized cache
  implementation("com.apollographql.apollo:apollo-normalized-cache-sqlite:x.y.z")
  // optional: for coroutines support
  implementation("com.apollographql.apollo:apollo-coroutines-support:x.y.z")
  // optional: for RxJava3 support  
  implementation("com.apollographql.apollo:apollo-rx3-support:x.y.z")
  // optional: if you just want the generated models and parsers and write your own HTTP code/cache code, you can remove apollo-runtime
  // and use apollo-api instead  
  implementation("com.apollographql.apollo:apollo-api:x.y.z")
}
```

Latest development changes are available in Sonatype's snapshots repository:

```gradle:title=build.gradle.kts
repositories {
  maven { 
    url = uri("https://oss.sonatype.org/content/repositories/snapshots/")
  }
}
```

## Requirements 

Apollo Android runs on the following platforms:

* Android API level 15+
* JDK 8+
* iOS 13+
 
## Contributing

If you'd like to contribute, please see [Contributing.md](https://github.com/apollographql/apollo-android/blob/master/Contributing.md).

## Additional resources

- [A journey to Kotlin multiplatform](https://www.youtube.com/watch?v=GN6LHrqyimI): how the project was moved to Kotlin multiplatform, talk given at Kotliners in June 2020.
- [#125, Fragmented Podcast](http://fragmentedpodcast.com/episodes/125/): Why's and How's about Apollo Android and the entire journey.
- [GraphQL.org](http://graphql.org) for an introduction and reference to GraphQL itself.
- [apollographql.com](http://www.apollographql.com/) to learn about Apollo open-source and commercial tools.
- [The Apollo blog](https://www.apollographql.com/blog/) for long-form articles about GraphQL, feature announcements for Apollo, and guest articles from the community.
- [The Apollo Twitter account](https://twitter.com/apollographql) for in-the-moment news.

## License

```
The MIT License (MIT)

Copyright (c) 2020 Meteor Development Group, Inc.
```
